//=======================================
// RTC_DS1202.c
//
// - beim ProgrammStart stets RTC_Init() aufrufen
// - mit RTC_SetTime() RTC setzen
// - mit RTC_GetTime() RTC lesen
// - die Structur von uhr ist
// - struct fm_timer { BYTE sec;
//                     BYTE min;                    
//                     BYTE hrs;
//                     BYTE day;
//                     BYTE month;
//                     BYTE year;
//                     BYTE dow;                  // day of week Montag = 1
//                   };
//=======================================
#include "ds1202.h"

//**************************************
// BCDtoBin
//**************************************
BYTE BCDtoBin(BYTE bcd)
{
	return ((bcd >> 4)*10+(bcd & 0x0F));
}
//**************************************
// BintoBCD
//**************************************
BYTE BintoBCD(BYTE dec)
{
    BYTE temp;
    temp = (dec / 10) << 4;
    temp = temp | (dec % 10);
    return temp; 
}
//**************************************
// Write            
//**************************************
static void Write( char cmd )
{ register BYTE n;

  RTC_DATA_LINE_IS_OUTPUT();
  for (n = 0; n < 8; n++)
  { if (CheckBit(cmd,n))
       RTC_DATA_HIGH();
    else  
       RTC_DATA_LOW();
    RTC_CLK_HIGH();
    NOP();
    NOP();
    RTC_CLK_LOW();
    NOP();
    NOP();
  }
  RTC_DATA_LINE_IS_INPUT();
}

//**************************************
// Write_DS1302
//**************************************
static void Write_DS1302( BYTE cmd, BYTE data )
{
  RTC_RESET_HIGH();
  Write(cmd);
  Write(data);
  RTC_RESET_LOW();
}

//***************************************
// Read
//***************************************
static BYTE Read( void )
{ 
   register BYTE n, d, data;
   data = 0x00;
   for(n = 0; n < 8; n++)
   {
     d = CHECK_RTC_DATA();
     data = (data >> 1);
     if (d)
       data |= 0x80;            // Store a '1'
     RTC_CLK_HIGH();
     NOP();
     NOP();
     RTC_CLK_LOW();
     NOP();
     NOP();
   }
   return(data);
}

//***************************************
// Read_DS1302
//***************************************
static BYTE Read_DS1302( BYTE cmd )
{
  BYTE data;
  
  RTC_RESET_HIGH();
  Write(cmd);
  data = Read();
  RTC_RESET_LOW();
  return data;
}

//*************************
// RTC_SetTime
//*************************
void RTC_SetTime( void )
{ 
  Write_DS1302(CONTROL_REGISTER, UNPROTECT);
  
  RTC_RESET_HIGH();
  Write( 0xBE );               // Clock-Write-Burst-Mode
  Write( BintoBCD(uhr.sec));
  Write( BintoBCD(uhr.min));
  Write( BintoBCD(uhr.hrs));
  Write( BintoBCD(uhr.day));
  Write( BintoBCD(uhr.month));
  Write( BintoBCD(uhr.dow) );              // Montag == 1   / Sonntag == 7        
  Write( BintoBCD(uhr.year));
  Write( UNPROTECT );
  RTC_RESET_LOW();

  Write_DS1302(CONTROL_REGISTER, PROTECT);
}

//*************************
// RTC_GetTime
//*************************
void RTC_GetTime( void )
{ BYTE buf[8];
  register BYTE n;
  
  RTC_RESET_HIGH();
  Write( 0xBF );             // Clock-Read-Burst-Mode
  for (n = 0; n < 8; n++)    // last read: Control-Register
     buf[n] = BCDtoBin(Read());
  uhr.sec   = buf[0];
  uhr.min   = buf[1];
  uhr.hrs   = buf[2];
  uhr.day   = buf[3];
  uhr.month = buf[4];
  uhr.dow   = buf[5];
  uhr.year  = buf[6];

  RTC_RESET_LOW();
   
}

//*************************
// RTC_Init         (called from Main-Init)      
//*************************
void RTC_Init(void )
{ BYTE data;

  RTC_RESET_LOW();
  RTC_DATA_LINE_IS_INPUT();
  NOP();
  NOP();
  RTC_CLK_LOW();
  NOP();
  NOP();

  Write_DS1302(CONTROL_REGISTER, UNPROTECT);
  data = Read_DS1302( 0x81);
  if (CheckBit(data,7))               // if bit 7 is high, Clock is stopped
  {  Write_DS1302( 0x80, 0x00);
  }
  Write_DS1302(CONTROL_REGISTER, PROTECT);

  RTC_GetTime();
}
